local mapgenutil = require "terrain/mapgenutil"
local treegen = require "V10lator/treegen"
local layersutil = require "terrain/layersutil"
local maputil = require "maputil"

function data() 

return {
	climate = "temperate.clima.lua",
	order = 0,
	name = _("Alps"),
	params = { },
	updateFn = function(params)
		-- local temperateassetsgen = dofile("./res/scripts/terrain/temperateassetsgen.lua")
		-- local mapgenutil = dofile("./res/scripts/terrain/mapgenutil.lua")
		-- local layersutil = dofile("./res/scripts/terrain/layersutil.lua")
		local result = {
			parallelFactor = 32,
			heightmapLayer = "HM",
			layers = layersutil.Layer.new(),
		}

		local sameSeed = math.random(1, 100000000)
		math.randomseed(sameSeed) -- reset seed to match desert generator
		
		local rivers = {}
	
		local ridgesConfig = {
			bounds = params.bounds, 
			probabilityLow = 0.03,
			probabilityHigh = 0.03, 
			minHeight = 1502, 
			maxHeight = 4697,
		}

		local valleys = {}
		local ridges = mapgenutil.MakeRidges(ridgesConfig)
		
		-- #################
		-- #### PREPARE
		local mkTemp = layersutil.TempMaker.new()
		
		-- #################
		-- #### BACKGROUND AND RIVER
		result.layers:Constant(result.heightmapLayer, .01)
		
		local noiseMap = mkTemp:Get()
		result.layers:Noise(noiseMap, 100)
		
		local distanceMap = mkTemp:Get()
		result.layers:Distance(result.heightmapLayer, distanceMap)
		
		-- #################
		-- #### RIDGES
		local ridgesMap
		do -- ridges
			result.layers:PushColor("#22AADD")
			local t1 = mkTemp:Get()
			result.layers:Map(distanceMap, t1, {15, 2500}, {0, 1}, true)
			result.layers:Mad(t1, noiseMap, result.heightmapLayer)
			noiseMap = mkTemp:Restore(noiseMap)
			
			local t2 = mkTemp:Get()
			result.layers:Ridge(t2, {
				valleys = valleys.points,
				ridges = ridges,
				noiseStrength = 10
			})
			
			result.layers:Map(distanceMap, t1, {50, 1500}, {0, 1}, true)
			ridgesMap = mkTemp:Get()
			result.layers:Mul(t1, t2, ridgesMap)
			t1 = mkTemp:Restore(t1)
			t2 = mkTemp:Restore(t2)

			result.layers:Add(ridgesMap, result.heightmapLayer, result.heightmapLayer)
			result.layers:PopColor()
		end
		
		-- #################
		-- #### NOISE
		local noiseStrength = 25.7
		local addNoise = true
		if addNoise then
			result.layers:PushColor("#5577DD")
			local t2 = mkTemp:Get()
			result.layers:RidgedNoise(t2, { octaves = 5, frequency = 1 / 444, lacunarity = 2.2, gain = 0.8 } )
			result.layers:Map(t2, t2, {0, 4}, {0, noiseStrength * 1.2}, false)
			
			local t1 = mkTemp:Get()
			result.layers:Map(distanceMap, t1, {0, 30}, {0, 1}, true)
			
			result.layers:Mad(t2, t1, result.heightmapLayer)
			mkTemp:Restore(t1)
			mkTemp:Restore(t2)
			result.layers:PopColor()
		end
		
		-- #################
		-- #### ASSETS
		result.layers:PushColor("#007777")
		result.forestMap, result.treesMapping, result.assetsMap, result.assetsMapping = treegen.Make(
			result.layers, mkTemp, result.heightmapLayer, ridgesMap, distanceMap
		)
		result.layers:PopColor()
		distanceMap = nil
	
		-- #################
		-- #### FINISH
		ridgesMap = mkTemp:Restore(ridgesMap)
		mkTemp:Restore(result.forestMap)
		mkTemp:Restore(result.assetsMap)
		mkTemp:Finish()
		-- maputil.PrintGraph(result)
	
		return result
	end
}

end
